import pandas as pd
import cv2
import csv
import os
import math
import time

class Manual:
    '''
    Each Manual instance is either a dyad (load_specific) or all (load_batch)
    '''
    def __init__(self, path):
        self.input_path = path # Path is the main directory, i.e. /Users/keshavrastogi/Downloads/IPD_Manual
        self.dyad_number = None

    # load in the csv
    def load_batch(self, analyze_type='all', use_existing_one=True):
        print("Loading all available")
        for filename in os.listdir(f"{self.input_path}/videos"):
            output_csv = filename.replace(".mp4", "_ipd.csv").replace("manual_vid_", "")
            if not os.path.exists(f"{self.input_path}/processed/{output_csv}"):
                if analyze_type == "per face":
                    print("Analyzing per face")
                    self.analyze_per_face(f"{self.input_path}/videos/{filename}", use_existing_one)
                else:
                    print("Analzying both faces")
                    self.analyze_all(f"{self.input_path}/videos/{filename}")

    def load_specific(self, dyad_num, analyze_type='all', use_existing_one=True):
        print(f"Loading dyad{dyad_num}")
        self.dyad_number = dyad_num

        for input_csv in os.listdir(f"{self.input_path}/input"):
            if f"_dyad{dyad_num}_" in input_csv and "csv" in input_csv:
                input_c = os.path.join(f"{self.input_path}/input", input_csv)
                output_csv = input_c.replace("input", "processed").replace("manual_csv_", "").replace(f"dyad{dyad_num}", f"dyad{dyad_num}_manual_ipd")

                if not os.path.exists(output_csv):
                    self.load_analyze(input_c, analyze_type, use_existing_one)
                else:
                    override = input(f"Dyad{dyad_num} already processed, Override (y/n): ")
                    if override in ['y', 'yes']:
                        os.remove(output_csv)
                        self.load_analyze(input_c, analyze_type, use_existing_one)


    def load_analyze(self, input, analyze_type, use_existing_one):
        if analyze_type == "per face":
            print("Analzying per face")
            self.analyze_per_face(input, use_existing_one)
        else:
            print("Analyzing both faces")
            self.analyze_all(input)

    def analyze_all(self, input):
        input_csv = input
        input_vid = input_csv.replace(".csv", ".mp4").replace("_csv_", "_vid_")
        print(f"Loading input: {input_vid}")
        output_csv = input.replace("input", "processed").replace("manual_csv_", "").replace(f"dyad{self.dyad_number}", f"dyad{self.dyad_number}_manual_ipd")
        output_vid = output_csv.replace(".csv", "_vid.mp4")

        df = pd.read_csv(input_csv)
        trial_number = df['Trial Number'].iloc[0]

        cap = cv2.VideoCapture(input_vid)
        if not cap.isOpened():
            print("Error: Could not open video.")
            exit()

        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')
        out = cv2.VideoWriter(output_vid, fourcc, 1, (width, height))

        current_index = 0

        while True: # Iterates through each frame in video
            ret, frame = cap.read() 
            if not ret:
                print("End of video")
                break

            points=[]
            status = ["Select first face"]
            first_face_confirmed =[False]
            clicked = [False]
            cv2.namedWindow('Manual Selection')
            cv2.setMouseCallback('Manual Selection', self.mouse_callback_all, (points, status, first_face_confirmed, clicked))

            while True:
                temp_frame = frame.copy()
                if len(points) >= 1:
                    cv2.rectangle(temp_frame, (points[0][0] - 50, points[0][1] - 50), (points[0][0] + 50, points[0][1] + 50), (0, 255, 0), 2)
                if len(points) >= 2:
                    cv2.rectangle(temp_frame, (points[1][0] - 50, points[1][1] - 50), (points[1][0] + 50, points[1][1] + 50), (0, 255, 0), 2)

                self.draw_text_and_rectangle(10, height-15, temp_frame, status[0])
                seconds = df['Second'].iloc[current_index]
                self.draw_text_and_rectangle(width-250, height-15, temp_frame, f"Second: {seconds}")
                self.draw_text_and_rectangle(width // 2 - 150, 30, temp_frame, f"Dyad {self.dyad_number}, Trial {trial_number}")
                cv2.imshow('Manual Selection', temp_frame)

                key = cv2.waitKey(1)
                # if key & 0xFF == 13:  # Enter key
                if key & clicked[0]:
                    if not first_face_confirmed[0] and len(points) == 1:
                        first_face_confirmed[0] = True
                        status[0] = "Select second face"
                    elif first_face_confirmed[0] and len(points) == 2:
                        status[0] = "Confirmed"
                        break
                    clicked[0] = False
                elif key & 0xFF == ord('q'):
                    self.exit(out, cap, output_csv, output_vid, self.dyad_number)
                    return

            bounding_boxes = [(points[0][0] - 50, points[0][1] - 50, 100, 100),
                            (points[1][0] - 50, points[1][1] - 50, 100, 100)]
            centers = [(x + w // 2, y + h // 2) for (x, y, w, h) in bounding_boxes]

            if len(centers) == 2:
                distance = int(math.sqrt((centers[0][0] - centers[1][0]) ** 2 + (centers[0][1] - centers[1][1]) ** 2))
                file_exists = os.path.isfile(output_csv)
                
                with open(output_csv, mode='a', newline='') as csv_file:
                    csv_writer = csv.writer(csv_file)
                    if not file_exists:
                        csv_writer.writerow(['Dyad Number', 'Trial Number', 'Second', 'X1', 'Y1', 'W1', 'H1', 'X2','Y2', 'W2', 'H2', 'Distance'])

                    csv_writer.writerow([self.dyad_number, trial_number, seconds, centers[0][0], centers[0][1], bounding_boxes[0][2], bounding_boxes[0][3], 
                                                 centers[1][0], centers[1][1], bounding_boxes[1][2], bounding_boxes[1][3], distance])

                for (x, y, w, h) in bounding_boxes:
                    cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)

                for center_x, center_y in centers:
                    cv2.circle(frame, (center_x, center_y), 5, (0, 0, 255), -1)
                cv2.line(frame, centers[0], centers[1], (255, 0, 0), 2)
                mid_x = (centers[0][0] + centers[1][0]) // 2
                mid_y = (centers[0][1] + centers[1][1]) // 2
                cv2.putText(frame, f"{distance}px", (mid_x, mid_y), cv2.FONT_HERSHEY_SIMPLEX, 0.75, (255, 255, 255), 2)
                self.draw_text_and_rectangle(width-250, height-15, frame, f"Second: {seconds}")
                self.draw_text_and_rectangle(width // 2 - 150, 30, frame, f"Dyad {self.dyad_number}, Trial {trial_number}") 

            out.write(frame)
            cv2.destroyWindow('Manual Selection')
            current_index += 1

        out.release()
        cap.release()
        cv2.destroyAllWindows()

    def mouse_callback_all(self, event, x, y, flags, param):
        points, status, first_face_confirmed, clicked = param
        if event == cv2.EVENT_LBUTTONDOWN:
            clicked[0] = True

        if event == cv2.EVENT_RBUTTONDOWN:
            if len(points) > 0:
                print('HI')
                points.clear()
                status[0] = "Select first face"
                first_face_confirmed[0] = False
                clicked[0] = False

        if not first_face_confirmed[0]:
            if len(points) < 1:
                points.append((x, y))
            else:
                points[0] = (x, y)
        elif first_face_confirmed[0]:
            if len(points) < 2:
                points.append((x, y))
            else:
                points[1] = (x, y)


    def exit(self, out, cap, output_csv, output_vid, study_number):
        out.release()
        cap.release()
        cv2.destroyAllWindows()
        if os.path.exists(output_csv):
            os.remove(output_csv)
        if os.path.exists(output_vid):
            os.remove(output_vid)
        print(f"Succesfully exited and deleted progress for Dyad{study_number}")

    def analyze_per_face(self, input, use_one):
        input_csv = input
        input_vid = input_csv.replace(".csv", ".mp4").replace("_csv_", "_vid_")
        print(f"Loading input: {input_vid}")
        output_csv = input.replace("input", "processed").replace("manual_csv_", "").replace(f"dyad{self.dyad_number}", f"dyad{self.dyad_number}_manual_ipd")
        output_vid = output_csv.replace(".csv", "_vid.mp4")

        df = pd.read_csv(input_csv)
        trial_number = df['Trial Number'].iloc[0]

        cap = cv2.VideoCapture(input_vid)
        if not cap.isOpened():
            print("Error: Could not open video.")
            exit()

        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')
        out = cv2.VideoWriter(output_vid, fourcc, 1, (width, height))
        total_frames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
        fps = cap.get(cv2.CAP_PROP_FPS)

        duration = total_frames / fps
        print(f"Manually calculating {duration} seconds")

        current_index = 0
        current_frame = 0
        points = [] # points[frame number][face number][X or Y]
        num_rows = len(df) # Number of outer list elements
        num_cols = 2  # Number of inner lists per outer element
        points = [[(0, 0, 0, 0) for _ in range(num_cols)] for _ in range(num_rows)]

        clicked = [False]

        while True: # First loop, going to evaluate each frame
            ret, frame = cap.read() 
            if not ret:
                print("End of first face processing")
                break
            clicked = [False]
            
            if pd.isna(df.loc[current_index, "X1"]) or not use_one:
                cv2.namedWindow(f'Manual Selection: Face {current_frame+1}')
                cv2.setMouseCallback(f'Manual Selection: Face {current_frame+1}', self.mouse_callback_per_face, (points, clicked, current_index, current_frame))
                
                while True:
                    temp_frame = frame.copy()
                    if points[current_index][0] != (0, 0, 0, 0):
                        x = points[current_index][0][0]
                        y = points[current_index][0][1]
                        w = points[current_index][0][2]
                        h = points[current_index][0][3]
                        cv2.rectangle(temp_frame, (x, y), (x + w, y + h), (0, 255, 0), 2)

                    self.draw_text_and_rectangle(10, height-15, temp_frame, "Select first face")
                    seconds = df['Second'].iloc[current_index]
                    self.draw_text_and_rectangle(width-250, height-15, temp_frame, f"Second: {seconds}")
                    self.draw_text_and_rectangle(width // 2 - 150, 30, temp_frame, f"Dyad {self.dyad_number}, Trial {trial_number}")

                    cv2.imshow(f'Manual Selection: Face {current_frame+1}', temp_frame)
                        
                    key = cv2.waitKey(1)
                    if key & clicked[0]:
                        break
                    elif key & 0xFF == ord('q'):
                        self.exit(out, cap, output_csv, output_vid, self.dyad_number)
                        return
                cv2.destroyWindow(f'Manual Selection: Face {current_frame+1}')
            elif use_one:
                x = int(df.loc[current_index, "X"])
                y = int(df.loc[current_index, "Y"])
                w = int(df.loc[current_index, "W"])
                h = int(df.loc[current_index, "H"])
                
                points[current_index][0] = (x, y, w, h)
            current_index += 1

        cv2.destroyAllWindows()
        current_index = 0
        current_frame = 1
        clicked[0] = False
        time.sleep(1.5)
        cap = cv2.VideoCapture(input_vid)
        if not cap.isOpened():
            print("Error: Could not open video.")
            exit()

        while True: # Second loop, going to evaluate each frame
            ret, frame = cap.read() 
            if not ret:   
                print("End of dyad processing")
                break

            clicked = [False]
        
            cv2.namedWindow(f'Manual Selection: Face {current_frame+1}')
            cv2.setMouseCallback(f'Manual Selection: Face {current_frame+1}', self.mouse_callback_per_face, (points, clicked, current_index, current_frame))
            
            while True:
                temp_frame = frame.copy()
                x = points[current_index][0][0]
                y = points[current_index][0][1]
                w = points[current_index][0][2]
                h = points[current_index][0][3]
                cv2.rectangle(temp_frame, (x, y), (x + w, y + h), (0, 255, 0), 2)

                x = points[current_index][1][0]
                y = points[current_index][1][1]
                w = points[current_index][1][2]
                h = points[current_index][1][3]
                cv2.rectangle(temp_frame, (x, y), (x + w, y + h), (0, 255, 0), 2)

                self.draw_text_and_rectangle(10, height-15, temp_frame, "Select second face")
                seconds = df['Second'].iloc[current_index]
                self.draw_text_and_rectangle(width-250, height-15, temp_frame, f"Second: {seconds}")
                self.draw_text_and_rectangle(width // 2 - 150, 30, temp_frame, f"Dyad {self.dyad_number}, Trial {trial_number}")

                cv2.imshow(f'Manual Selection: Face {current_frame+1}', temp_frame)
                    
                key = cv2.waitKey(1)
                # if key & 0xFF == 13:  # Enter key
                if key & clicked[0]:
                    break
                elif key & 0xFF == ord('q'):
                    self.exit(out, cap, output_csv, output_vid, self.dyad_number)
                    return

            # calculation shit
            bounding_boxes = [(points[current_index][0][0], points[current_index][0][1], points[current_index][0][2], points[current_index][0][3]),
                            (points[current_index][1][0], points[current_index][1][1], points[current_index][1][2], points[current_index][1][3])]
            centers = [(x + w // 2, y + h // 2) for (x, y, w, h) in bounding_boxes]

            if len(centers) == 2:
                distance = int(math.sqrt((centers[0][0] - centers[1][0]) ** 2 + (centers[0][1] - centers[1][1]) ** 2))
                file_exists = os.path.isfile(output_csv)

                with open(output_csv, mode='a', newline='') as csv_file:
                    csv_writer = csv.writer(csv_file)
                    if not file_exists:
                        csv_writer.writerow(['Dyad Number', 'Trial Number', 'Second', 'X1', 'Y1', 'W1', 'H1', 'X2','Y2', 'W2', 'H2', 'Distance'])

                    csv_writer.writerow([self.dyad_number, trial_number, seconds, centers[0][0], centers[0][1], bounding_boxes[0][2], bounding_boxes[0][3], 
                                                 centers[1][0], centers[1][1], bounding_boxes[1][2], bounding_boxes[1][3], distance])
                for (x, y, w, h) in bounding_boxes:
                    cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)

                for center_x, center_y in centers:
                    cv2.circle(frame, (center_x, center_y), 5, (0, 0, 255), -1)
                cv2.line(frame, centers[0], centers[1], (255, 0, 0), 2)
                mid_x = (centers[0][0] + centers[1][0]) // 2
                mid_y = (centers[0][1] + centers[1][1]) // 2
                cv2.putText(frame, f"{distance}px", (mid_x, mid_y), cv2.FONT_HERSHEY_SIMPLEX, 0.75, (255, 255, 255), 2)

                self.draw_text_and_rectangle(width-250, height-15, frame, f"Second: {seconds}")
                self.draw_text_and_rectangle(width // 2 - 150, 30, frame, f"Dyad {self.dyad_number}, Trial {trial_number}")

            out.write(frame)
            cv2.destroyWindow(f'Manual Selection: Face {current_frame+1}')
            current_index += 1

        out.release()
        cap.release()
        cv2.destroyAllWindows()

    def mouse_callback_per_face(self, event, x, y, flags, param):
        points, clicked, current_index, current_frame = param
        if event == cv2.EVENT_RBUTTONDOWN and current_frame == 1:
            points[current_index][0] = (x, y, 100, 100)

        if event == cv2.EVENT_LBUTTONDOWN:
            clicked[0] = True

        points[current_index][current_frame] = (x, y, 100, 100)
    
    def draw_text_and_rectangle(self, x, y, frame, text):
        font_face = cv2.FONT_HERSHEY_SIMPLEX
        font_scale = 1.25
        thickness = 4
        
        text_size, _ = cv2.getTextSize(text, font_face, font_scale, thickness)
        text_width, text_height = text_size

        text_org = (x, y)
        top_left = (x - 10, y - text_height - 10)
        bottom_right = (x + text_width + 10, y + 10)

        cv2.rectangle(frame, top_left, bottom_right, (0, 0, 0), cv2.FILLED) # black rectangle
        cv2.putText(frame, text, text_org, font_face, font_scale, (255, 255, 255), thickness, cv2.LINE_AA) # white text

def process_file():
    path = "/Users/VHILab Core/Documents/FIDAC/Manual_Tracking"
    manu = Manual(path)
    study_num = 0
    manu.load_specific(study_num, analyze_type='per face', use_existing_one=False)

if __name__ == "__main__":
    process_file()


''' THIS IS FOR FIDAC '''

from imports import *
from pyfeat_detector import PyFeat_Detection

import logging
logging.getLogger().setLevel(logging.ERROR) # this is to prevent py-feat's NO FACE DETECTED error, which doesn't matter

# Code that runs py-feat or other libraries

# need to consider best way to set up build 

# also add total seconds (i.e. Second: 23/156)

class Distance:
    '''
    Each distance object is a specific trial (i.e. trial 1 or trial2) of a specific study
    '''

    def __init__(self, input):
        self.video_path = input

    def compute_distance(self, num_faces=2, second_frequency=1, detection_model='py-feat', optimize=False):
        '''
        Determines location of faces, computes and stores interpersonal distance, loads manual frames.
        Currently utlizies py-feat's Detector with a face_thershold of .9
        '''
        ''' add check to see if dyad file in file is same as the folder'''
        self.study_number = int(''.join(filter(str.isdigit, os.path.basename(self.video_path).split('_')[0][4:]))) # determines study_number from string manipulation
        self.trial_number = 1 if 'trial1' in self.video_path else 2 # gets trial number
        self.num_faces = num_faces
        ''' need to change trial so it fits however many the user wants'''

        input_video_path = self.video_path
        core_folder = input_video_path.split('/Input/')[0]

        video_output_path = f'{core_folder}/Processed/dyad{self.study_number}/video_dyad{self.study_number}_trial{self.trial_number}.mp4'
        temp_frame_path = core_folder + '/temp_frame.jpg'
        csv_output_path = video_output_path.replace("video_", "csv_").replace(".mp4", ".csv")

        print(f"Calculating distance from: {input_video_path}")
        if os.path.exists(video_output_path) and not optimize: # check if already exists
            override = input("File already processed: Override? (y/n): ")
            if override == "n" or override == "no":
                return
            
        # set up the model to use for detection
        if detection_model == 'py-feat': 
            print("Using py-feat")
            model = PyFeat_Detection()
        elif detection_model == 'mediapipe': 
            print("Using mediapipe")
            from mediapipe_detector import MediaPipe_Detection
            model = MediaPipe_Detection()
        else:
            print("Incorrect model selected")

        # important frame information captured by cv2
        cap = cv2.VideoCapture(input_video_path)
        fps = int(cap.get(cv2.CAP_PROP_FPS))
        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')
        total_frames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
        duration = int(total_frames / fps)

        # output_video, which is at 1 frame per second
        out = cv2.VideoWriter(video_output_path, fourcc, 1, (width, height)) 

        frame_count = 0 # current frame on
        distance = None # distance between faces
        manual_frames = [] # list of frames not detected, saved as (second, frame)
        manual_values = [] # info about manual frames, saved as (second, x, y, w, h)
        seconds_tracked = []

        if optimize and os.path.exists(video_output_path):
            # saves the path to a csv and video path created as temp files
            csv_old, video_path_old = setup_prev_model()

        with open(csv_output_path, mode='w', newline='') as csv_file:
            csv_writer = csv.writer(csv_file)
            if detection_model != 'mediapipe': pbar = tqdm(total=duration)
           
            # x, y are the points' center
            headers = ['Study Number', 'Trial Number', 'Second']
            for i in range(1, num_faces + 1):
                headers.extend([f'X{i}', f'Y{i}', f'W{i}', f'H{i}'])
            headers.append('Distance')
            csv_writer.writerow(headers) # first row output

            if optimize:
                # adds previous values to the CSV
                self.csv_add_prev_values()

            while cap.isOpened(): # iterates through each frame
                ret, frame = cap.read()
                if not ret:
                    if optimize:
                        self.vid_add_prev_values()
                    print("Can't receive frame (stream end?). Exiting ...")
                    break
                
                if frame_count % int(fps / second_frequency) == 0: # only evaluates a certain amount of times
                    if detection_model != 'mediapipe': pbar.update(1)
                    seconds = frame_count // fps
                    centers = [] # center of face, stored as (x, y)

                    # creates a temporary frame and detects faces on that
                    cv2.imwrite(temp_frame_path, frame)

                    filtered_detections, bounding_boxes = model.calculate_bounding_box(temp_frame_path, num_faces)

                    if len(bounding_boxes) == num_faces: # only stores in csv if exact amt of faces
                        centers = [(x + w // 2, y + h // 2) for (x, y, w, h) in bounding_boxes]
                        distance = int(math.sqrt((centers[0][0] - centers[1][0]) ** 2 + (centers[0][1] - centers[1][1]) ** 2))
                        csv_writer.writerow([self.study_number, self.trial_number, seconds, centers[0][0], centers[0][1], bounding_boxes[0][2], bounding_boxes[0][3], 
                                                centers[1][0], centers[1][1], bounding_boxes[1][2], bounding_boxes[1][3], distance])
                        seconds_tracked.append(seconds)
                    # change this        
                    elif len(filtered_detections) == 1: # if only one face is detected, appends to manual_values
                        for x, y, w, h in bounding_boxes:
                            manual_values.append((seconds, x, y, w, h))
                        manual_frames.append((frame, seconds))
                        bounding_boxes.clear()
                    else:
                        manual_frames.append((frame, seconds))

                    # draws boxes around faces
                    for (x, y, w, h) in bounding_boxes: 
                        cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)

                    if len(centers) == 2 and distance is not None:
                        # draw dots around center of face and line connecting them with distance in pixels written
                        for x, y in centers:
                            cv2.circle(frame, (x, y), 5, (0, 0, 255), -1)
                        cv2.line(frame, centers[0], centers[1], (255, 0, 0), 2)
                        x_mid = (centers[0][0] + centers[1][0]) // 2
                        y_mid = (centers[0][1] + centers[1][1]) // 2
                        cv2.putText(frame, f"{distance}px", (x_mid, y_mid), cv2.FONT_HERSHEY_SIMPLEX, 0.75, (255, 255, 255), 2)

                        # output seconds and dyad information
                        self.draw_text_and_rectangle(width-250, height-15, frame, f"Second: {seconds}")
                        self.draw_text_and_rectangle(width // 2 - 150, 30, frame, f"Study {self.study_number}, Trial {self.trial_number}")

                        out.write(frame)

                frame_count += 1
            if detection_model != 'mediapipe': pbar.close()
        
        self.create_manual_video(manual_frames, manual_values)

        out.release()
        cap.release()
        cv2.destroyAllWindows()

        os.remove(temp_frame_path)

    def draw_text_and_rectangle(self, x, y, frame, text):
        '''
        Draws text at specified x,y location on a specific frame.
        Based on length of text (in white), draws a black rectangle surroudning it
        '''
        font_face = cv2.FONT_HERSHEY_SIMPLEX
        font_scale = 1.25
        thickness = 4
        
        text_size, _ = cv2.getTextSize(text, font_face, font_scale, thickness)
        text_width, text_height = text_size

        text_org = (x, y)
        top_left = (x - 10, y - text_height - 10)
        bottom_right = (x + text_width + 10, y + 10)

        cv2.rectangle(frame, top_left, bottom_right, (0, 0, 0), cv2.FILLED) # black rectangle
        cv2.putText(frame, text, text_org, font_face, font_scale, (255, 255, 255), thickness, cv2.LINE_AA) # white text

    def setup_prev_model(self):
        '''
        Creates a temporary csv and video with the output from the previous model
        
        '''
        pass


    def csv_add_prev_values(self):
        '''
        adds the saved input_values from the old model and merges with new model's output
        sorts through them and delets overlaps (favors new model?)
        '''
        pass
    
    def vid_add_prev_values(self):
        '''
        adds the saved video frames from the old model to the new model. doesn't care about sorting
        '''
        pass

    def create_manual_video(self, manual_frames, manual_values):
        '''
        Creates csv and video of all manual frames that need to be human-coded.

        Key variables:

        '''
        
        # create manual file
        core_folder = self.video_path.split('/Input/')[0]
        manual_path = core_folder + '/Manual_Tracking'
        os.makedirs(manual_path + '/input', exist_ok=True)
        os.makedirs(manual_path + '/processed', exist_ok=True)

        csv_manual_path = f'{manual_path}/input/manual_csv_dyad{self.study_number}_trial{self.trial_number}.csv'
        video_manual_path = csv_manual_path.replace("_csv", "_vid").replace(".csv", '.mp4')

        if not manual_frames: # checks if no frames to add manually
            print("No manual frames")
            return
        
        height, width, layers = manual_frames[0][0].shape
        fourcc = cv2.VideoWriter_fourcc(*'mp4v') 
        
        video = cv2.VideoWriter(video_manual_path, fourcc, 1, (width, height)) # 1 frame per second
        
        # opens new file, removing the one that currently exists
        with open(csv_manual_path, mode='w', newline='') as csv_file:
            csv_writer = csv.writer(csv_file)

            m_headers = ['Study Number', 'Trial Number', 'Second']
            for i in range(1, self.num_faces):
                m_headers.extend([f'X{i}', f'Y{i}', f'W{i}', f'H{i}'])
            csv_writer.writerow(m_headers)

            '''
            Iterates through each frame in manual_frames, noting the current second. It then loops through 
            manual values, and if that second is also apparant in manual values, the row includes: x, y, w, h.
            If not, it just includes the study_number, trial_number, and second
            '''
            for frame, second in manual_frames:
                found = False
                for sec, x, y, w, h in manual_values:
                    if sec == second:
                        csv_writer.writerow([self.study_number, self.trial_number, second, x, y, w, h])
                        found = True
                if not found:
                    csv_writer.writerow([self.study_number, self.trial_number, second])

                # output seconds and dyad information
                self.draw_text_and_rectangle(width-250, height-15, frame, f"Second: {second}")
                self.draw_text_and_rectangle(width // 2 - 150, 30, frame, f"Study {self.study_number}, Trial {self.trial_number}")
                video.write(frame) 
        video.release()

def process_file(num, model, opt):
    core_path = f'/Users/VHILab Core/Documents/FIDAC' # this needs to be changed per user

    input_path = core_path + f'/Input/dyad{num}/'
    os.makedirs(input_path.replace("Input", "Processed"), exist_ok=True)

    if not os.path.exists(input_path):
        print(f"'{input_path}' does not exist.")
    else:
        for filename in os.listdir(input_path):
            if filename.lower().endswith(".mp4") and "center" in filename.lower():
                name = input_path + filename
                dist = Distance(name) # name includes everything till the file
                dist.compute_distance(optimize, detection_model=model, optimize=opt)

def rename_file(file_path):
    directory, original_file_name = os.path.split(file_path)
    new_file_name = original_file_name.lower().replace(" ", "_")
    new_file_path = os.path.join(directory, new_file_name)
    os.rename(file_path, new_file_path)
    return new_file_path

if __name__ == "__main__":
    optimize = False
    if len(sys.argv) == 3:
        study_num = int(sys.argv[1])
        model_type = sys.argv[2]
    if len(sys.argv) > 3:
        if sys.argv[3] == "optimize": optimize = True

    process_file(study_num, model_type, optimize)



'''
Similar to manual_calculate, this file goes through a set of outputed models and plays them back relatively fast.
Users then have the option of pressing enter for the next or R to continue to the next video
'''
from imports import *


class Eval_Output:
    def process_output(self, path, dyad_num, trial_num):
        '''
        Write code that opens a video, whose path is stored in the variable 'path'. 
        The video is saved in 1 fps but I want to play it faster, so make it by default 10 fps and have the speed saved in an adjustable variable.
        If the user presses space, the video pauses and if they press it again it plays. Add some text in the bottom that says "Press space to play/pause". 
        After the video is complete, text should appear on the video (in the middle) saying: "Repeat (r) or End (enter)". If the user presses r,
        the video repeats but this time is in the original 1 fps speed. If they press enter, then the video closes
        '''
        video = cv2.VideoCapture(path)
        if not video.isOpened():
            print("Error: Could not open video.")
            exit()

        # playback speeds
        original_fps = 1
        playback_fps = 5
        current_fps = playback_fps

        # Playback control variables
        paused = True 
        started = False

        ret, frame = video.read()
        if not ret:
            print("Error: Could not read the first frame of the video.")
            exit()

        self.display_text(frame, "Press 's' to start", (50, int(video.get(cv2.CAP_PROP_FRAME_HEIGHT) // 2)), scale=1.5)
        cv2.imshow('Video Playback', frame)

        while True:
            # Wait for the user to press 's' to start the video
            if not started:
                key = cv2.waitKey(0)
                if key == ord('s'):
                    started = True
                    paused = False
                elif key == ord('q'):
                    break
                continue

            if not paused:
                ret, frame = video.read()
                if not ret:
                    # End of video
                    if frame_prev is not None:
                        # Use the last frame to display repeat/exit options
                        frame = frame_prev.copy()
                        self.display_text(frame, "Repeat (r) or End (q)", (50, int(video.get(cv2.CAP_PROP_FRAME_HEIGHT) // 2)), scale=1.5)
                        cv2.imshow('Video Playback', frame)
                        
                        key = cv2.waitKey(0)  # Wait for user input
                        if key == ord('r'):
                            print(f"Flagging dyad {dyad_num} trial {trial_num}")
                            current_fps = original_fps
                            video.set(cv2.CAP_PROP_POS_FRAMES, 0)  # Restart the video
                        elif key == ord('q'): 
                            break
                    else:
                        print("Error: No frames available to display.")
                        break
                    continue

                frame_prev = frame.copy()  # Save last frame to use when the video ends
                self.display_text(frame, "Press space to play/pause", (10, int(video.get(cv2.CAP_PROP_FRAME_HEIGHT)) - 20))
                cv2.imshow('Video Playback', frame)

            key = cv2.waitKey(int(1000 / current_fps))

            if key == ord(' '):  # Space bar to pause/play
                paused = not paused
            elif key == ord('q'):  # Quit the video if 'q' is pressed
                break

        video.release()
        cv2.destroyAllWindows()


    # Function to display text on frame\
    def display_text(self, frame, text, position, scale=1, color=(255, 255, 255)):
        text_size, _ = cv2.getTextSize(text, cv2.FONT_HERSHEY_SIMPLEX, scale, 2)
        text_width, text_height = text_size

        x, y = position
        box_coords = ((x, y - text_height - 10), (x + text_width + 10, y + 10))

        cv2.rectangle(frame, box_coords[0], box_coords[1], (0, 0, 0), cv2.FILLED)
        cv2.putText(frame, text, (x, y), cv2.FONT_HERSHEY_SIMPLEX, scale, color, 2, cv2.LINE_AA)

def process_file():
    study_range = [2, 10]
    for study in range(study_range[0], study_range[1]+1):
        if study != 4:
            for trial_num in range (1, 3): 
                path = f'/Users/VHILab Core/Documents/VHIL_E2E_Test/Output/dyad{study}/ipd_video_trial{trial_num}.mp4'
                eval = Eval_Output()
                eval.process_output(path, study, trial_num)


if __name__ == "__main__":
    process_file()


import subprocess
path = "/Users/VHILab Core/Documents/FIDAC/Programs/measure_position.py"
subprocess.run(["python", path, "300", "py-feat"])
subprocess.run(["python", path, "300", "mediapipe", "optimize"])

'''
Mediapipe: 3, 4, 8, 9, 11, 13, 14
py-feat: got 7 right but missing 11

'''


from imports import *
import mediapipe as mp

class MediaPipe_Detection:
    def calculate_bounding_box(self, temp_path, number_faces, confidence=0.7):
        '''
        Input needed: temp_frame_path, num_faces
        
        '''
        mp_face_detection = mp.solutions.face_detection

        bounding_boxes = []
        filtered_detections = []
        image = cv2.imread(temp_path)
        height, width, _ = image.shape

        with mp_face_detection.FaceDetection(model_selection=1, min_detection_confidence=confidence) as face_detection:
            image_rgb = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
            results = face_detection.process(image_rgb)

            if results.detections:
                for detection in results.detections:
                    bboxC = detection.location_data.relative_bounding_box
                    x, y, w, h = int(bboxC.xmin * width), int(bboxC.ymin * height), int(bboxC.width * width), int(bboxC.height * height)

                    if w > 50 and h > 50:
                        filtered_detections.append({
                            'FaceRectX': x, 
                            'FaceRectY': y, 
                            'FaceRectWidth': w, 
                            'FaceRectHeight': h, 
                            'FaceScore': detection.score[0]  
                        })

        filtered_detections = sorted(filtered_detections, key=lambda x: x['FaceScore'], reverse=True)[:number_faces]

        for r in filtered_detections:
            bounding_boxes.append((r['FaceRectX'], r['FaceRectY'], r['FaceRectWidth'], r['FaceRectHeight']))
        filtered_detections_df = pd.DataFrame(filtered_detections)
        
        return filtered_detections_df, bounding_boxes


from imports import *
from feat import Detector
from feat.utils.io import read_feat

def silent_tqdm(*args, **kwargs):
    return tqdm(*args, **kwargs, disable=True)

import feat.detector
feat.detector.tqdm = silent_tqdm

class PyFeat_Detection:
    def calculate_bounding_box(self, temp_path, number_faces, confidence=.9):
        '''
        Input needed: temp_frame_path, num_faces
        
        '''
        detector = Detector(
            face_model='retinaface',
            landmark_model='mobilefacenet',
            au_model='xgb',
            emotion_model='resmasknet',
            facepose_model='img2pose',
            device='cpu',
            face_threshold=confidence
        )
        bounding_boxes = [] # box around rectangle stored as (x, y, w, h)
        detections = detector.detect_image(temp_path, )

        filtered_detections = []
        for index, row in detections.iterrows():
            if not pd.isna(row['FaceRectX']) and not pd.isna(row['FaceRectY']) and not pd.isna(row['FaceRectWidth']) and not pd.isna(row['FaceRectHeight']):
                x, y, w, h = int(row['FaceRectX']), int(row['FaceRectY']), int(row['FaceRectWidth']), int(row['FaceRectHeight'])
                if w > 50 and h > 50: # only adds 'faces' larger than 50 x 5, needs to be determined manually
                    filtered_detections.append(row)

        # only saves num_faces number of faces, ordered based on FaceScore (confidence)
        filtered_detections = sorted(filtered_detections, key=lambda x: x['FaceScore'], reverse=True)[:number_faces]

        for r in filtered_detections:
            bounding_boxes.append((int(r['FaceRectX']), int(r['FaceRectY']), int(r['FaceRectWidth']), int(r['FaceRectHeight'])))
        
        return filtered_detections, bounding_boxes


from setuptools import setup, find_packages

setup(
    name='fidac',
    version='0.1.0',
    author='Keshav Rastogi',
    author_email='keshavrast21@gmail.com',
    description='insert description', # modify later
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/yourusername/my_library', # modify later
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    license='MIT',
    python_requires='>=3.9',
    install_requires=[
        'pandas',
        'opencv-python',  # for cv2
        'tqdm',
        'numpy',
        'matplotlib',
        'seaborn',
    ]
)


__version__ = '0.1.0'


import pandas as pd
import cv2
import csv
from tqdm import tqdm
import os
import subprocess
import math
import numpy as np
import matplotlib.pyplot as plt
from glob import glob
import seaborn as sns
import re
import sys
import time
from setuptools import setup, find_packages

__all__ = ['pd', 'cv2', 'csv', 'tqdm', 'os', 'subprocess', 'math', 'np', 'plt', 'glob', 'sns', 're', 'sys', 'time', 'setuptools']

import logging
logging.getLogger(__name__).addHandler(logging.NullHandler())